# Copyright 2017 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
from unittest import mock

from osc_lib.tests import utils as osctestutils

from ironicclient import exc
from ironicclient.osc.v1 import baremetal_volume_target as bm_vol_target
from ironicclient.tests.unit.osc.v1 import fakes as baremetal_fakes


class TestBaremetalVolumeTarget(baremetal_fakes.TestBaremetal):

    def setUp(self):
        super(TestBaremetalVolumeTarget, self).setUp()

        self.baremetal_mock = self.app.client_manager.baremetal
        self.baremetal_mock.reset_mock()


class TestCreateBaremetalVolumeTarget(TestBaremetalVolumeTarget):

    def setUp(self):
        super(TestCreateBaremetalVolumeTarget, self).setUp()

        self.baremetal_mock.volume_target.create.return_value = (
            baremetal_fakes.FakeBaremetalResource(
                None,
                copy.deepcopy(baremetal_fakes.VOLUME_TARGET),
                loaded=True,
            ))

        # Get the command object to test
        self.cmd = (
            bm_vol_target.CreateBaremetalVolumeTarget(self.app, None))

    def test_baremetal_volume_target_create(self):
        arglist = [
            '--node', baremetal_fakes.baremetal_uuid,
            '--type',
            baremetal_fakes.baremetal_volume_target_volume_type,
            '--boot-index',
            baremetal_fakes.baremetal_volume_target_boot_index,
            '--volume-id',
            baremetal_fakes.baremetal_volume_target_volume_id,
            '--uuid', baremetal_fakes.baremetal_volume_target_uuid,
        ]

        verifylist = [
            ('node_uuid', baremetal_fakes.baremetal_uuid),
            ('volume_type',
             baremetal_fakes.baremetal_volume_target_volume_type),
            ('boot_index',
             baremetal_fakes.baremetal_volume_target_boot_index),
            ('volume_id',
             baremetal_fakes.baremetal_volume_target_volume_id),
            ('uuid', baremetal_fakes.baremetal_volume_target_uuid),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        args = {
            'node_uuid': baremetal_fakes.baremetal_uuid,
            'volume_type':
                baremetal_fakes.baremetal_volume_target_volume_type,
            'boot_index':
                baremetal_fakes.baremetal_volume_target_boot_index,
            'volume_id':
                baremetal_fakes.baremetal_volume_target_volume_id,
            'uuid': baremetal_fakes.baremetal_volume_target_uuid,
        }

        self.baremetal_mock.volume_target.create.assert_called_once_with(
            **args)

    def test_baremetal_volume_target_create_without_uuid(self):
        arglist = [
            '--node', baremetal_fakes.baremetal_uuid,
            '--type',
            baremetal_fakes.baremetal_volume_target_volume_type,
            '--boot-index',
            baremetal_fakes.baremetal_volume_target_boot_index,
            '--volume-id',
            baremetal_fakes.baremetal_volume_target_volume_id,
        ]

        verifylist = [
            ('node_uuid', baremetal_fakes.baremetal_uuid),
            ('volume_type',
             baremetal_fakes.baremetal_volume_target_volume_type),
            ('boot_index',
             baremetal_fakes.baremetal_volume_target_boot_index),
            ('volume_id',
             baremetal_fakes.baremetal_volume_target_volume_id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        args = {
            'node_uuid': baremetal_fakes.baremetal_uuid,
            'volume_type':
                baremetal_fakes.baremetal_volume_target_volume_type,
            'boot_index':
                baremetal_fakes.baremetal_volume_target_boot_index,
            'volume_id':
                baremetal_fakes.baremetal_volume_target_volume_id,
        }

        self.baremetal_mock.volume_target.create.assert_called_once_with(
            **args)

    def test_baremetal_volume_target_create_extras(self):
        arglist = [
            '--node', baremetal_fakes.baremetal_uuid,
            '--type',
            baremetal_fakes.baremetal_volume_target_volume_type,
            '--boot-index',
            baremetal_fakes.baremetal_volume_target_boot_index,
            '--volume-id',
            baremetal_fakes.baremetal_volume_target_volume_id,
            '--extra', 'key1=value1',
            '--extra', 'key2=value2',
        ]

        verifylist = [
            ('node_uuid', baremetal_fakes.baremetal_uuid),
            ('volume_type',
             baremetal_fakes.baremetal_volume_target_volume_type),
            ('boot_index',
             baremetal_fakes.baremetal_volume_target_boot_index),
            ('volume_id',
             baremetal_fakes.baremetal_volume_target_volume_id),
            ('extra', ['key1=value1', 'key2=value2'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        args = {
            'node_uuid': baremetal_fakes.baremetal_uuid,
            'volume_type':
                baremetal_fakes.baremetal_volume_target_volume_type,
            'boot_index':
                baremetal_fakes.baremetal_volume_target_boot_index,
            'volume_id':
                baremetal_fakes.baremetal_volume_target_volume_id,
            'extra': baremetal_fakes.baremetal_volume_target_extra,
        }

        self.baremetal_mock.volume_target.create.assert_called_once_with(
            **args)

    def _test_baremetal_volume_target_missing_param(self, missing):
        argdict = {
            '--node': baremetal_fakes.baremetal_uuid,
            '--type':
                baremetal_fakes.baremetal_volume_target_volume_type,
            '--boot-index':
                baremetal_fakes.baremetal_volume_target_boot_index,
            '--volume-id':
                baremetal_fakes.baremetal_volume_target_volume_id,
            '--uuid': baremetal_fakes.baremetal_volume_target_uuid,
        }

        arglist = []
        for k, v in argdict.items():
            if k not in missing:
                arglist += [k, v]
        verifylist = None

        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_create_missing_node(self):
        self._test_baremetal_volume_target_missing_param(['--node'])

    def test_baremetal_volume_target_create_missing_type(self):
        self._test_baremetal_volume_target_missing_param(['--type'])

    def test_baremetal_volume_target_create_missing_boot_index(self):
        self._test_baremetal_volume_target_missing_param(['--boot-index'])

    def test_baremetal_volume_target_create_missing_volume_id(self):
        self._test_baremetal_volume_target_missing_param(['--volume-id'])

    def test_baremetal_volume_target_create_invalid_boot_index(self):
        arglist = [
            '--node', baremetal_fakes.baremetal_uuid,
            '--type',
            baremetal_fakes.baremetal_volume_target_volume_type,
            '--boot-index', 'string',
            '--volume-id',
            baremetal_fakes.baremetal_volume_target_volume_id,
        ]
        verifylist = None

        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_create_negative_boot_index(self):
        arglist = [
            '--node', baremetal_fakes.baremetal_uuid,
            '--type',
            baremetal_fakes.baremetal_volume_target_volume_type,
            '--boot-index', '-1',
            '--volume-id',
            baremetal_fakes.baremetal_volume_target_volume_id,
        ]

        verifylist = [
            ('node_uuid', baremetal_fakes.baremetal_uuid),
            ('volume_type',
             baremetal_fakes.baremetal_volume_target_volume_type),
            ('boot_index', -1),
            ('volume_id',
             baremetal_fakes.baremetal_volume_target_volume_id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(exc.CommandError, self.cmd.take_action, parsed_args)


class TestShowBaremetalVolumeTarget(TestBaremetalVolumeTarget):

    def setUp(self):
        super(TestShowBaremetalVolumeTarget, self).setUp()

        self.baremetal_mock.volume_target.get.return_value = (
            baremetal_fakes.FakeBaremetalResource(
                None,
                copy.deepcopy(baremetal_fakes.VOLUME_TARGET),
                loaded=True))

        self.cmd = (
            bm_vol_target.ShowBaremetalVolumeTarget(self.app, None))

    def test_baremetal_volume_target_show(self):
        arglist = ['vvv-tttttt-vvvv']
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['vvv-tttttt-vvvv']
        self.baremetal_mock.volume_target.get.assert_called_once_with(
            *args, fields=None)
        collist = ('boot_index', 'extra', 'node_uuid', 'properties', 'uuid',
                   'volume_id', 'volume_type')
        self.assertEqual(collist, columns)

        datalist = (
            baremetal_fakes.baremetal_volume_target_boot_index,
            baremetal_fakes.baremetal_volume_target_extra,
            baremetal_fakes.baremetal_uuid,
            baremetal_fakes.baremetal_volume_target_properties,
            baremetal_fakes.baremetal_volume_target_uuid,
            baremetal_fakes.baremetal_volume_target_volume_id,
            baremetal_fakes.baremetal_volume_target_volume_type,
        )
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_show_no_options(self):
        arglist = []
        verifylist = []
        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_show_fields(self):
        arglist = ['vvv-tttttt-vvvv', '--fields', 'uuid', 'volume_id']
        verifylist = [('fields', [['uuid', 'volume_id']]),
                      ('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid)]

        fake_vt = copy.deepcopy(baremetal_fakes.VOLUME_TARGET)
        fake_vt.pop('node_uuid')
        fake_vt.pop('volume_type')
        fake_vt.pop('boot_index')
        fake_vt.pop('extra')
        fake_vt.pop('properties')
        self.baremetal_mock.volume_target.get.return_value = (
            baremetal_fakes.FakeBaremetalResource(
                None,
                fake_vt,
                loaded=True))

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['vvv-tttttt-vvvv']
        fields = ['uuid', 'volume_id']
        self.baremetal_mock.volume_target.get.assert_called_once_with(
            *args, fields=fields)
        collist = ('uuid', 'volume_id')
        self.assertEqual(collist, columns)

        datalist = (
            baremetal_fakes.baremetal_volume_target_uuid,
            baremetal_fakes.baremetal_volume_target_volume_id,
        )
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_show_fields_multiple(self):
        arglist = ['vvv-tttttt-vvvv', '--fields', 'uuid', 'volume_id',
                   '--fields', 'volume_type']
        verifylist = [('fields', [['uuid', 'volume_id'], ['volume_type']]),
                      ('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid)]

        fake_vt = copy.deepcopy(baremetal_fakes.VOLUME_TARGET)
        fake_vt.pop('node_uuid')
        fake_vt.pop('boot_index')
        fake_vt.pop('extra')
        fake_vt.pop('properties')
        self.baremetal_mock.volume_target.get.return_value = (
            baremetal_fakes.FakeBaremetalResource(
                None,
                fake_vt,
                loaded=True))

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        args = ['vvv-tttttt-vvvv']
        fields = ['uuid', 'volume_id', 'volume_type']
        self.baremetal_mock.volume_target.get.assert_called_once_with(
            *args, fields=fields)
        collist = ('uuid', 'volume_id', 'volume_type')
        self.assertEqual(collist, columns)

        datalist = (
            baremetal_fakes.baremetal_volume_target_uuid,
            baremetal_fakes.baremetal_volume_target_volume_id,
            baremetal_fakes.baremetal_volume_target_volume_type,
        )
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_show_invalid_fields(self):
        arglist = ['vvv-tttttt-vvvv', '--fields', 'uuid', 'invalid']
        verifylist = None

        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)


class TestListBaremetalVolumeTarget(TestBaremetalVolumeTarget):
    def setUp(self):
        super(TestListBaremetalVolumeTarget, self).setUp()

        self.baremetal_mock.volume_target.list.return_value = [
            baremetal_fakes.FakeBaremetalResource(
                None,
                copy.deepcopy(baremetal_fakes.VOLUME_TARGET),
                loaded=True)
        ]
        self.cmd = (
            bm_vol_target.ListBaremetalVolumeTarget(self.app, None))

    def test_baremetal_volume_target_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        kwargs = {
            'marker': None,
            'limit': None}
        self.baremetal_mock.volume_target.list.assert_called_once_with(
            **kwargs)

        collist = (
            "UUID",
            "Node UUID",
            "Driver Volume Type",
            "Boot Index",
            "Volume ID")
        self.assertEqual(collist, columns)

        datalist = ((baremetal_fakes.baremetal_volume_target_uuid,
                     baremetal_fakes.baremetal_uuid,
                     baremetal_fakes.baremetal_volume_target_volume_type,
                     baremetal_fakes.baremetal_volume_target_boot_index,
                     baremetal_fakes.baremetal_volume_target_volume_id),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_list_node(self):
        arglist = ['--node', baremetal_fakes.baremetal_uuid]
        verifylist = [('node', baremetal_fakes.baremetal_uuid)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        kwargs = {
            'node': baremetal_fakes.baremetal_uuid,
            'marker': None,
            'limit': None}
        self.baremetal_mock.volume_target.list.assert_called_once_with(
            **kwargs)

        collist = (
            "UUID",
            "Node UUID",
            "Driver Volume Type",
            "Boot Index",
            "Volume ID")
        self.assertEqual(collist, columns)

        datalist = ((baremetal_fakes.baremetal_volume_target_uuid,
                     baremetal_fakes.baremetal_uuid,
                     baremetal_fakes.baremetal_volume_target_volume_type,
                     baremetal_fakes.baremetal_volume_target_boot_index,
                     baremetal_fakes.baremetal_volume_target_volume_id),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_list_long(self):
        arglist = ['--long']
        verifylist = [('detail', True)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        kwargs = {
            'detail': True,
            'marker': None,
            'limit': None,
        }
        self.baremetal_mock.volume_target.list.assert_called_with(**kwargs)

        collist = ('UUID', 'Node UUID', 'Driver Volume Type', 'Properties',
                   'Boot Index', 'Extra', 'Volume ID', 'Created At',
                   'Updated At')
        self.assertEqual(collist, columns)

        datalist = ((baremetal_fakes.baremetal_volume_target_uuid,
                     baremetal_fakes.baremetal_uuid,
                     baremetal_fakes.baremetal_volume_target_volume_type,
                     baremetal_fakes.baremetal_volume_target_properties,
                     baremetal_fakes.baremetal_volume_target_boot_index,
                     baremetal_fakes.baremetal_volume_target_extra,
                     baremetal_fakes.baremetal_volume_target_volume_id,
                     '',
                     ''),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_list_fields(self):
        arglist = ['--fields', 'uuid', 'boot_index']
        verifylist = [('fields', [['uuid', 'boot_index']])]

        fake_vt = copy.deepcopy(baremetal_fakes.VOLUME_TARGET)
        fake_vt.pop('volume_type')
        fake_vt.pop('extra')
        fake_vt.pop('properties')
        fake_vt.pop('volume_id')
        fake_vt.pop('node_uuid')
        self.baremetal_mock.volume_target.list.return_value = [
            baremetal_fakes.FakeBaremetalResource(
                None,
                fake_vt,
                loaded=True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        kwargs = {
            'detail': False,
            'marker': None,
            'limit': None,
            'fields': ('uuid', 'boot_index')
        }
        self.baremetal_mock.volume_target.list.assert_called_with(**kwargs)

        collist = ('UUID', 'Boot Index')
        self.assertEqual(collist, columns)

        datalist = ((baremetal_fakes.baremetal_volume_target_uuid,
                     baremetal_fakes.baremetal_volume_target_boot_index),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_list_fields_multiple(self):
        arglist = ['--fields', 'uuid', 'boot_index', '--fields', 'extra']
        verifylist = [('fields', [['uuid', 'boot_index'], ['extra']])]

        fake_vt = copy.deepcopy(baremetal_fakes.VOLUME_TARGET)
        fake_vt.pop('volume_type')
        fake_vt.pop('properties')
        fake_vt.pop('volume_id')
        fake_vt.pop('node_uuid')
        self.baremetal_mock.volume_target.list.return_value = [
            baremetal_fakes.FakeBaremetalResource(
                None,
                fake_vt,
                loaded=True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        kwargs = {
            'detail': False,
            'marker': None,
            'limit': None,
            'fields': ('uuid', 'boot_index', 'extra')
        }
        self.baremetal_mock.volume_target.list.assert_called_with(**kwargs)

        collist = ('UUID', 'Boot Index', 'Extra')
        self.assertEqual(collist, columns)

        datalist = ((baremetal_fakes.baremetal_volume_target_uuid,
                     baremetal_fakes.baremetal_volume_target_boot_index,
                     baremetal_fakes.baremetal_volume_target_extra),)
        self.assertEqual(datalist, tuple(data))

    def test_baremetal_volume_target_list_invalid_fields(self):
        arglist = ['--fields', 'uuid', 'invalid']
        verifylist = [('fields', [['uuid', 'invalid']])]
        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_list_marker(self):
        arglist = ['--marker', baremetal_fakes.baremetal_volume_target_uuid]
        verifylist = [
            ('marker', baremetal_fakes.baremetal_volume_target_uuid)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        kwargs = {
            'marker': baremetal_fakes.baremetal_volume_target_uuid,
            'limit': None}
        self.baremetal_mock.volume_target.list.assert_called_once_with(
            **kwargs)

    def test_baremetal_volume_target_list_limit(self):
        arglist = ['--limit', '10']
        verifylist = [('limit', 10)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        kwargs = {
            'marker': None,
            'limit': 10}
        self.baremetal_mock.volume_target.list.assert_called_once_with(
            **kwargs)

    def test_baremetal_volume_target_list_sort(self):
        arglist = ['--sort', 'boot_index']
        verifylist = [('sort', 'boot_index')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        kwargs = {
            'marker': None,
            'limit': None}
        self.baremetal_mock.volume_target.list.assert_called_once_with(
            **kwargs)

    def test_baremetal_volume_target_list_sort_desc(self):
        arglist = ['--sort', 'boot_index:desc']
        verifylist = [('sort', 'boot_index:desc')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        kwargs = {
            'marker': None,
            'limit': None}
        self.baremetal_mock.volume_target.list.assert_called_once_with(
            **kwargs)

    def test_baremetal_volume_target_list_exclusive_options(self):
        arglist = ['--fields', 'uuid', '--long']
        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, [])

    def test_baremetal_volume_target_list_negative_limit(self):
        arglist = ['--limit', '-1']
        verifylist = [('limit', -1)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exc.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestDeleteBaremetalVolumeTarget(TestBaremetalVolumeTarget):

    def setUp(self):
        super(TestDeleteBaremetalVolumeTarget, self).setUp()

        self.cmd = bm_vol_target.DeleteBaremetalVolumeTarget(self.app, None)

    def test_baremetal_volume_target_delete(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid]
        verifylist = [('volume_targets',
                       [baremetal_fakes.baremetal_volume_target_uuid])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        self.baremetal_mock.volume_target.delete.assert_called_with(
            baremetal_fakes.baremetal_volume_target_uuid)

    def test_baremetal_volume_target_delete_multiple(self):
        fake_volume_target_uuid2 = 'vvv-tttttt-tttt'
        arglist = [baremetal_fakes.baremetal_volume_target_uuid,
                   fake_volume_target_uuid2]
        verifylist = [('volume_targets',
                       [baremetal_fakes.baremetal_volume_target_uuid,
                        fake_volume_target_uuid2])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)

        self.baremetal_mock.volume_target.delete.assert_has_calls(
            [mock.call(baremetal_fakes.baremetal_volume_target_uuid),
             mock.call(fake_volume_target_uuid2)])
        self.assertEqual(
            2, self.baremetal_mock.volume_target.delete.call_count)

    def test_baremetal_volume_target_delete_no_options(self):
        arglist = []
        verifylist = []
        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_delete_error(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid]
        verifylist = [('volume_targets',
                       [baremetal_fakes.baremetal_volume_target_uuid])]

        self.baremetal_mock.volume_target.delete.side_effect = (
            exc.NotFound())

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exc.ClientException,
                          self.cmd.take_action,
                          parsed_args)
        self.baremetal_mock.volume_target.delete.assert_called_with(
            baremetal_fakes.baremetal_volume_target_uuid)

    def test_baremetal_volume_target_delete_multiple_error(self):
        fake_volume_target_uuid2 = 'vvv-tttttt-tttt'
        arglist = [baremetal_fakes.baremetal_volume_target_uuid,
                   fake_volume_target_uuid2]
        verifylist = [('volume_targets',
                       [baremetal_fakes.baremetal_volume_target_uuid,
                        fake_volume_target_uuid2])]

        self.baremetal_mock.volume_target.delete.side_effect = [
            None, exc.NotFound()]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exc.ClientException,
                          self.cmd.take_action,
                          parsed_args)

        self.baremetal_mock.volume_target.delete.assert_has_calls(
            [mock.call(baremetal_fakes.baremetal_volume_target_uuid),
             mock.call(fake_volume_target_uuid2)])
        self.assertEqual(
            2, self.baremetal_mock.volume_target.delete.call_count)


class TestSetBaremetalVolumeTarget(TestBaremetalVolumeTarget):
    def setUp(self):
        super(TestSetBaremetalVolumeTarget, self).setUp()

        self.cmd = (
            bm_vol_target.SetBaremetalVolumeTarget(self.app, None))

    def test_baremetal_volume_target_set_node_uuid(self):
        new_node_uuid = 'xxx-xxxxxx-zzzz'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--node', new_node_uuid]
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('node_uuid', new_node_uuid)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/node_uuid', 'value': new_node_uuid, 'op': 'add'}])

    def test_baremetal_volume_target_set_volume_type(self):
        new_type = 'fibre_channel'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--type', new_type]
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('volume_type', new_type)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/volume_type', 'value': new_type, 'op': 'add'}])

    def test_baremetal_volume_target_set_boot_index(self):
        new_boot_idx = '3'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--boot-index', new_boot_idx]
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('boot_index', int(new_boot_idx))]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/boot_index', 'value': int(new_boot_idx), 'op': 'add'}])

    def test_baremetal_volume_target_set_negative_boot_index(self):
        new_boot_idx = '-3'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--boot-index', new_boot_idx]
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('boot_index', int(new_boot_idx))]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(exc.CommandError, self.cmd.take_action, parsed_args)

    def test_baremetal_volume_target_set_invalid_boot_index(self):
        new_boot_idx = 'string'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--boot-index', new_boot_idx]
        verifylist = None

        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_set_volume_id(self):
        new_volume_id = 'new-volume-id'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--volume-id', new_volume_id]
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('volume_id', new_volume_id)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/volume_id', 'value': new_volume_id, 'op': 'add'}])

    def test_baremetal_volume_target_set_volume_type_and_volume_id(self):
        new_volume_type = 'fibre_channel'
        new_volume_id = 'new-volume-id'
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--type', new_volume_type,
            '--volume-id', new_volume_id]
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('volume_type', new_volume_type),
            ('volume_id', new_volume_id)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/volume_type', 'value': new_volume_type, 'op': 'add'},
             {'path': '/volume_id', 'value': new_volume_id, 'op': 'add'}])

    def test_baremetal_volume_target_set_extra(self):
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--extra', 'foo=bar']
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('extra', ['foo=bar'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/extra/foo', 'value': 'bar', 'op': 'add'}])

    def test_baremetal_volume_target_set_multiple_extras(self):
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--extra', 'key1=val1', '--extra', 'key2=val2']
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('extra', ['key1=val1', 'key2=val2'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/extra/key1', 'value': 'val1', 'op': 'add'},
             {'path': '/extra/key2', 'value': 'val2', 'op': 'add'}])

    def test_baremetal_volume_target_set_property(self):
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--property', 'foo=bar']
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('properties', ['foo=bar'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/properties/foo', 'value': 'bar', 'op': 'add'}])

    def test_baremetal_volume_target_set_multiple_properties(self):
        arglist = [
            baremetal_fakes.baremetal_volume_target_uuid,
            '--property', 'key1=val1', '--property', 'key2=val2']
        verifylist = [
            ('volume_target',
             baremetal_fakes.baremetal_volume_target_uuid),
            ('properties', ['key1=val1', 'key2=val2'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/properties/key1', 'value': 'val1', 'op': 'add'},
             {'path': '/properties/key2', 'value': 'val2', 'op': 'add'}])

    def test_baremetal_volume_target_set_no_options(self):
        arglist = []
        verifylist = []
        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_set_no_property(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid]
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_not_called()


class TestUnsetBaremetalVolumeTarget(TestBaremetalVolumeTarget):
    def setUp(self):
        super(TestUnsetBaremetalVolumeTarget, self).setUp()

        self.cmd = bm_vol_target.UnsetBaremetalVolumeTarget(self.app, None)

    def test_baremetal_volume_target_unset_extra(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid,
                   '--extra', 'key1']
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid),
                      ('extra', ['key1'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/extra/key1', 'op': 'remove'}])

    def test_baremetal_volume_target_unset_multiple_extras(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid,
                   '--extra', 'key1', '--extra', 'key2']
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid),
                      ('extra', ['key1', 'key2'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/extra/key1', 'op': 'remove'},
             {'path': '/extra/key2', 'op': 'remove'}])

    def test_baremetal_volume_target_unset_property(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid,
                   '--property', 'key11']
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid),
                      ('properties', ['key11'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/properties/key11', 'op': 'remove'}])

    def test_baremetal_volume_target_unset_multiple_properties(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid,
                   '--property', 'key11', '--property', 'key22']
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid),
                      ('properties', ['key11', 'key22'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_called_once_with(
            baremetal_fakes.baremetal_volume_target_uuid,
            [{'path': '/properties/key11', 'op': 'remove'},
             {'path': '/properties/key22', 'op': 'remove'}])

    def test_baremetal_volume_target_unset_no_options(self):
        arglist = []
        verifylist = []
        self.assertRaises(osctestutils.ParserException,
                          self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_baremetal_volume_target_unset_no_property(self):
        arglist = [baremetal_fakes.baremetal_volume_target_uuid]
        verifylist = [('volume_target',
                       baremetal_fakes.baremetal_volume_target_uuid)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.baremetal_mock.volume_target.update.assert_not_called()



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Red Hat, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.common import base
from ironicclient.common.i18n import _
from ironicclient.common import utils
from ironicclient import exc


class Port(base.Resource):
    def __repr__(self):
        return "<Port %s>" % self._info


class PortManager(base.CreateManager):
    resource_class = Port
    _creation_attributes = ['address', 'extra', 'local_link_connection',
                            'node_uuid', 'physical_network', 'portgroup_uuid',
                            'pxe_enabled', 'uuid', 'is_smartnic']
    _resource_name = 'ports'

    def list(self, address=None, limit=None, marker=None, sort_key=None,
             sort_dir=None, detail=False, fields=None, node=None,
             portgroup=None, os_ironic_api_version=None,
             global_request_id=None):
        """Retrieve a list of ports.

        :param address: Optional, MAC address of a port, to get
                       the port which has this MAC address
        :param marker: Optional, the UUID of a port, eg the last
                       port from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of ports to return.
            2) limit == 0, return the entire list of ports.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about ports.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param node: Optional, name or UUID of a node. Used to get
                     ports of this node.

        :param portgroup: Optional, name or UUID of a portgroup. Used to get
                          ports of this portgroup.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of ports.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)
        if address is not None:
            filters.append('address=%s' % address)
        if node is not None:
            filters.append('node=%s' % node)
        if portgroup is not None:
            filters.append('portgroup=%s' % portgroup)

        path = ''
        if detail:
            path += 'detail'
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "ports", **header_values)
        else:
            return self._list_pagination(self._path(path), "ports",
                                         limit=limit, **header_values)

    def get(self, port_id, fields=None, os_ironic_api_version=None,
            global_request_id=None):
        return self._get(resource_id=port_id, fields=fields,
                         os_ironic_api_version=os_ironic_api_version,
                         global_request_id=global_request_id)

    def get_by_address(self, address, fields=None, os_ironic_api_version=None,
                       global_request_id=None):
        path = '?address=%s' % address
        if fields is not None:
            path += '&fields=' + ','.join(fields)
        else:
            path = 'detail' + path

        ports = self._list(self._path(path), 'ports',
                           os_ironic_api_version=os_ironic_api_version,
                           global_request_id=global_request_id)
        # get all the details of the port assuming that filtering by
        # address returns a collection of one port if successful.
        if len(ports) == 1:
            return ports[0]
        else:
            raise exc.NotFound()

    def delete(self, port_id, os_ironic_api_version=None,
               global_request_id=None):
        return self._delete(resource_id=port_id,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)

    def update(self, port_id, patch, os_ironic_api_version=None,
               global_request_id=None):
        return self._update(resource_id=port_id, patch=patch,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ironicclient.tests.functional.osc.v1 import base


class BaremetalConductorTests(base.TestCase):
    """Functional tests for baremetal conductor commands."""

    def test_list(self):
        """List available conductors.

        There is at lease one conductor in the functional tests, if not, other
        tests will fail too.
        """
        hostnames = [c['Hostname'] for c in self.conductor_list()]
        self.assertIsNotNone(hostnames)

    def test_show(self):
        """Show specified conductor.

        Conductor name varies in different environment, list first, then show
        one of them.
        """
        conductors = self.conductor_list()
        conductor = self.conductor_show(conductors[0]['Hostname'])
        self.assertIn('conductor_group', conductor)
        self.assertIn('alive', conductor)
        self.assertIn('drivers', conductor)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ddt
from tempest.lib import exceptions

from ironicclient.tests.functional.osc.v1 import base


@ddt.ddt
class BaremetalNodeCreateNegativeTests(base.TestCase):
    """Negative tests for node create command."""

    def setUp(self):
        super(BaremetalNodeCreateNegativeTests, self).setUp()

    @ddt.data(
        ('--uuid', '', 'expected one argument'),
        ('--uuid', '!@#$^*&%^', 'Expected UUID for uuid'),
        ('--uuid', '0000 0000', 'unrecognized arguments'),
        ('--driver-info', '', 'expected one argument'),
        ('--driver-info', 'some info', 'unrecognized arguments'),
        ('--property', '', 'expected one argument'),
        ('--property', 'some property', 'unrecognized arguments'),
        ('--extra', '', 'expected one argument'),
        ('--extra', 'some extra', 'unrecognized arguments'),
        ('--name', '', 'expected one argument'),
        ('--name', 'some name', 'unrecognized arguments'),
        ('--network-interface', '', 'expected one argument'),
        ('--resource-class', '', 'expected one argument'))
    @ddt.unpack
    def test_baremetal_node_create(self, argument, value, ex_text):
        base_cmd = 'baremetal node create --driver %s' % self.driver_name
        command = self.construct_cmd(base_cmd, argument, value)
        self.assertRaisesRegex(exceptions.CommandFailed, ex_text,
                               self.openstack, command)



# Copyright 2016 SAP Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.common import base
from ironicclient.common.i18n import _
from ironicclient.common import utils
from ironicclient import exc


class Portgroup(base.Resource):
    def __repr__(self):
        return "<Portgroup %s>" % self._info


class PortgroupManager(base.CreateManager):
    resource_class = Portgroup
    _resource_name = 'portgroups'
    _creation_attributes = ['address', 'extra', 'name', 'node_uuid',
                            'standalone_ports_supported', 'mode', 'properties']

    def list(self, node=None, address=None, limit=None, marker=None,
             sort_key=None, sort_dir=None, detail=False, fields=None,
             os_ironic_api_version=None, global_request_id=None):
        """Retrieve a list of portgroups.

        :param node: Optional, UUID or name of a node, to get
                     the portgroups for that node.
        :param address: Optional, MAC address of a portgroup, to get
                       the portgroup which has this MAC address.
        :param marker: Optional, the UUID of a portgroup, eg the last
                       portgroup from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of portgroups to return.
            2) limit == 0, return the entire list of portgroups.
            3) limit == None, the number of items returned respect the
               maximum imposed by the Ironic API (see Ironic's
               api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about portgroups.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of portgroups.
        :raises: InvalidAttribute if a subset of fields is requested with
                 detail option set.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)
        if address is not None:
            filters.append('address=%s' % address)
        if node is not None:
            filters.append('node=%s' % node)

        path = ''
        if detail:
            path += 'detail'
        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "portgroups", **header_values)
        else:
            return self._list_pagination(self._path(path), "portgroups",
                                         limit=limit, **header_values)

    def list_ports(self, portgroup_id, marker=None, limit=None, sort_key=None,
                   sort_dir=None, detail=False, fields=None,
                   os_ironic_api_version=None, global_request_id=None):
        """List all the ports for a given portgroup.

        :param portgroup_id: Name or UUID of the portgroup.
        :param marker: Optional, the UUID of a port, eg the last
                       port from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of ports to return.
            2) limit == 0, return the entire list of ports.
            3) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the Ironic API
               (see Ironic's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :param detail: Optional, boolean whether to return detailed information
                       about ports.

        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.

        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.

        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: A list of ports.

        """
        if limit is not None:
            limit = int(limit)

        if detail and fields:
            raise exc.InvalidAttribute(_("Can't fetch a subset of fields "
                                         "with 'detail' set"))

        filters = utils.common_filters(marker, limit, sort_key, sort_dir,
                                       fields)

        path = "%s/ports" % portgroup_id
        if detail:
            path += '/detail'

        if filters:
            path += '?' + '&'.join(filters)
        header_values = {"os_ironic_api_version": os_ironic_api_version,
                         "global_request_id": global_request_id}
        if limit is None:
            return self._list(self._path(path), "ports", **header_values)
        else:
            return self._list_pagination(self._path(path), "ports",
                                         limit=limit, **header_values)

    def get(self, portgroup_id, fields=None, os_ironic_api_version=None,
            global_request_id=None):
        """Get a port group with the specified identifier.

        :param portgroup_id: The UUID or name of a portgroup.
        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: a :class:`Portgroup` object.

        """
        return self._get(resource_id=portgroup_id, fields=fields,
                         os_ironic_api_version=os_ironic_api_version,
                         global_request_id=global_request_id)

    def get_by_address(self, address, fields=None, os_ironic_api_version=None,
                       global_request_id=None):
        """Get a port group with the specified MAC address.

        :param address: The MAC address of a portgroup.
        :param fields: Optional, a list with a specified set of fields
                       of the resource to be returned. Can not be used
                       when 'detail' is set.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.

        :returns: a :class:`Portgroup` object.

        """
        path = '?address=%s' % address
        if fields is not None:
            path += '&fields=' + ','.join(fields)
        else:
            path = 'detail' + path

        portgroups = self._list(self._path(path), 'portgroups',
                                os_ironic_api_version=os_ironic_api_version,
                                global_request_id=global_request_id)
        # get all the details of the portgroup assuming that
        # filtering by address returns a collection of one portgroup
        # if successful.
        if len(portgroups) == 1:
            return portgroups[0]
        else:
            raise exc.NotFound()

    def delete(self, portgroup_id, os_ironic_api_version=None,
               global_request_id=None):
        """Delete the Portgroup from the DB.

        :param portgroup_id: The UUID or name of a portgroup.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        return self._delete(resource_id=portgroup_id,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)

    def update(self, portgroup_id, patch, os_ironic_api_version=None,
               global_request_id=None):
        """Update the Portgroup.

        :param portgroup_id: The UUID or name of a portgroup.
        :param patch: The patch request with updates.
        :param os_ironic_api_version: String version (e.g. "1.35") to use for
            the request.  If not specified, the client's default is used.
        :param global_request_id: String containing global request ID header
            value (in form "req-<UUID>") to use for the request.
        """
        return self._update(resource_id=portgroup_id, patch=patch,
                            os_ironic_api_version=os_ironic_api_version,
                            global_request_id=global_request_id)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import configparser
import os

from tempest.lib.cli import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

import ironicclient.tests.functional.utils as utils

DEFAULT_CONFIG_FILE = os.path.join(os.path.dirname(__file__), 'test.conf')


class FunctionalTestBase(base.ClientTestBase):
    """Ironic base class, calls to ironicclient."""

    def setUp(self):
        super(FunctionalTestBase, self).setUp()
        self.client = self._get_clients()
        # NOTE(kromanenko) set ironic api version for portgroups
        self.pg_api_ver = '--ironic-api-version 1.25'

    def _get_clients(self):
        # NOTE(aarefiev): {toxinidir} is a current working directory, so
        # the tox env path is {toxinidir}/.tox
        venv_name = os.environ.get('OS_TESTENV_NAME', 'functional')
        cli_dir = os.path.join(os.path.abspath('.'), '.tox/%s/bin' % venv_name)

        config = self._get_config()
        if config.get('os_auth_url'):
            client = base.CLIClient(cli_dir=cli_dir,
                                    username=config['os_username'],
                                    password=config['os_password'],
                                    tenant_name=config['os_project_name'],
                                    uri=config['os_auth_url'])
            for keystone_object in 'user', 'project':
                domain_attr = 'os_%s_domain_id' % keystone_object
                if config.get(domain_attr):
                    setattr(self, domain_attr, config[domain_attr])
        else:
            self.ironic_url = config['ironic_url']
            client = base.CLIClient(cli_dir=cli_dir,
                                    ironic_url=self.ironic_url)
        return client

    def _get_config(self):
        config_file = os.environ.get('IRONICCLIENT_TEST_CONFIG',
                                     DEFAULT_CONFIG_FILE)
        config = configparser.ConfigParser()
        if not config.read(config_file):
            self.skipTest('Skipping, no test config found @ %s' % config_file)
        try:
            auth_strategy = config.get('functional', 'auth_strategy')
        except configparser.NoOptionError:
            auth_strategy = 'keystone'
        if auth_strategy not in ['keystone', 'noauth']:
            raise self.fail(
                'Invalid auth type specified: %s in functional must be '
                'one of: [keystone, noauth]' % auth_strategy)

        conf_settings = []
        keystone_v3_conf_settings = []
        if auth_strategy == 'keystone':
            conf_settings += ['os_auth_url', 'os_username',
                              'os_password', 'os_project_name']
            keystone_v3_conf_settings += ['os_user_domain_id',
                                          'os_project_domain_id',
                                          'os_identity_api_version']
        else:
            conf_settings += ['ironic_url']

        cli_flags = {}
        missing = []
        for c in conf_settings + keystone_v3_conf_settings:
            try:
                cli_flags[c] = config.get('functional', c)
            except configparser.NoOptionError:
                # NOTE(vdrok): Here we ignore the absence of KS v3 options as
                # v2 may be used. Keystone client will do the actual check of
                # the parameters' correctness.
                if c not in keystone_v3_conf_settings:
                    missing.append(c)
        if missing:
            self.fail('Missing required setting in test.conf (%(conf)s) for '
                      'auth_strategy=%(auth)s: %(missing)s' %
                      {'conf': config_file,
                       'auth': auth_strategy,
                       'missing': ','.join(missing)})
        return cli_flags

    def _cmd_no_auth(self, cmd, action, flags='', params=''):
        """Execute given command with noauth attributes.

        :param cmd: command to be executed
        :type cmd: string
        :param action: command on cli to run
        :type action: string
        :param flags: optional cli flags to use
        :type flags: string
        :param params: optional positional args to use
        :type params: string
        """
        flags = ('--os-endpoint %(url)s %(flags)s'
                 %
                 {'url': self.ironic_url,
                  'flags': flags})
        return base.execute(cmd, action, flags, params,
                            cli_dir=self.client.cli_dir)

    def _ironic(self, action, cmd='ironic', flags='', params='',
                merge_stderr=False):
        """Execute ironic command for the given action.

        :param action: the cli command to run using Ironic
        :type action: string
        :param cmd: the base of cli command to run
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param merge_stderr: whether to merge stderr into the result
        :type merge_stderr: bool
        """
        if cmd == 'openstack':
            config = self._get_config()
            id_api_version = config.get('os_identity_api_version')
            if id_api_version:
                flags += ' --os-identity-api-version {}'.format(id_api_version)
        else:
            flags += ' --os-endpoint-type publicURL'

        if hasattr(self, 'ironic_url'):
            if cmd == 'openstack':
                flags += ' --os-auth-type none'
            return self._cmd_no_auth(cmd, action, flags, params)
        else:
            for keystone_object in 'user', 'project':
                domain_attr = 'os_%s_domain_id' % keystone_object
                if hasattr(self, domain_attr):
                    flags += ' --os-%(ks_obj)s-domain-id %(value)s' % {
                        'ks_obj': keystone_object,
                        'value': getattr(self, domain_attr)
                    }
            return self.client.cmd_with_auth(
                cmd, action, flags, params, merge_stderr=merge_stderr)

    def ironic(self, action, flags='', params='', parse=True):
        """Return parsed list of dicts with basic item info.

        :param action: the cli command to run using Ironic
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param parse: return parsed list or raw output
        :type parse: bool
        """
        output = self._ironic(action=action, flags=flags, params=params)
        return self.parser.listing(output) if parse else output

    def get_table_headers(self, action, flags='', params=''):
        output = self._ironic(action=action, flags=flags, params=params)
        table = self.parser.table(output)
        return table['headers']

    def assertTableHeaders(self, field_names, table_headers):
        """Assert that field_names and table_headers are equal.

        :param field_names: field names from the output table of the cmd
        :param table_headers: table headers output from cmd
        """
        self.assertEqual(sorted(field_names), sorted(table_headers))

    def assertNodeStates(self, node_show, node_show_states):
        """Assert that node_show_states output corresponds to node_show output.

        :param node_show: output from node-show cmd
        :param node_show_states: output from node-show-states cmd
        """
        for key in node_show_states.keys():
            self.assertEqual(node_show_states[key], node_show[key])

    def assertNodeValidate(self, node_validate):
        """Assert that all interfaces present are valid.

        :param node_validate: output from node-validate cmd
        """
        self.assertNotIn('False', [x['Result'] for x in node_validate])

    def delete_node(self, node_id):
        """Delete node method works only with fake driver.

        :param node_id: node uuid
        :raises: CommandFailed exception when command fails to delete a node
        """
        node_list = self.list_nodes()

        if utils.get_object(node_list, node_id):
            node_show = self.show_node(node_id)
            if node_show['provision_state'] not in ('available',
                                                    'manageable',
                                                    'enroll'):
                self.ironic('node-set-provision-state',
                            params='{0} deleted'.format(node_id))
            if node_show['power_state'] not in ('None', 'off'):
                self.ironic('node-set-power-state',
                            params='{0} off'.format(node_id))
            self.ironic('node-delete', params=node_id)

            node_list_uuid = self.get_nodes_uuids_from_node_list()
            if node_id in node_list_uuid:
                self.fail('Ironic node {0} has not been deleted!'
                          .format(node_id))

    def create_node(self, driver='fake-hardware', params=''):
        node = self.ironic('node-create',
                           params='--driver {0} {1}'.format(driver, params))

        if not node:
            self.fail('Ironic node has not been created!')

        node = utils.get_dict_from_output(node)
        self.addCleanup(self.delete_node, node['uuid'])
        return node

    def show_node(self, node_id, params=''):
        node_show = self.ironic('node-show',
                                params='{0} {1}'.format(node_id, params))
        return utils.get_dict_from_output(node_show)

    def list_nodes(self, params=''):
        return self.ironic('node-list', params=params)

    def update_node(self, node_id, params):
        updated_node = self.ironic('node-update',
                                   params='{0} {1}'.format(node_id, params))
        return utils.get_dict_from_output(updated_node)

    def get_nodes_uuids_from_node_list(self):
        node_list = self.list_nodes()
        return [x['UUID'] for x in node_list]

    def show_node_states(self, node_id):
        show_node_states = self.ironic('node-show-states', params=node_id)
        return utils.get_dict_from_output(show_node_states)

    def set_node_maintenance(self, node_id, maintenance_mode, params=''):
        self.ironic(
            'node-set-maintenance',
            params='{0} {1} {2}'.format(node_id, maintenance_mode, params))

    def set_node_power_state(self, node_id, power_state, params=''):
        self.ironic('node-set-power-state',
                    params='{0} {1} {2}'
                    .format(node_id, power_state, params))

    def set_node_provision_state(self, node_id, provision_state, params=''):
        self.ironic('node-set-provision-state',
                    params='{0} {1} {2}'
                    .format(node_id, provision_state, params))

    def validate_node(self, node_id):
        return self.ironic('node-validate', params=node_id)

    def list_node_chassis(self, chassis_uuid, params=''):
        return self.ironic('chassis-node-list',
                           params='{0} {1}'.format(chassis_uuid, params))

    def get_nodes_uuids_from_chassis_node_list(self, chassis_uuid):
        chassis_node_list = self.list_node_chassis(chassis_uuid)
        return [x['UUID'] for x in chassis_node_list]

    def list_driver(self, params=''):
        return self.ironic('driver-list', params=params)

    def show_driver(self, driver_name):
        driver_show = self.ironic('driver-show', params=driver_name)
        return utils.get_dict_from_output(driver_show)

    def properties_driver(self, driver_name):
        return self.ironic('driver-properties', params=driver_name)

    def get_drivers_names(self):
        driver_list = self.list_driver()
        return [x['Supported driver(s)'] for x in driver_list]

    def delete_chassis(self, chassis_id, ignore_exceptions=False):
        try:
            self.ironic('chassis-delete', params=chassis_id)
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def get_chassis_uuids_from_chassis_list(self):
        chassis_list = self.list_chassis()
        return [x['UUID'] for x in chassis_list]

    def create_chassis(self, params=''):
        chassis = self.ironic('chassis-create', params=params)

        if not chassis:
            self.fail('Ironic chassis has not been created!')

        chassis = utils.get_dict_from_output(chassis)
        self.addCleanup(self.delete_chassis,
                        chassis['uuid'],
                        ignore_exceptions=True)
        return chassis

    def list_chassis(self, params=''):
        return self.ironic('chassis-list', params=params)

    def show_chassis(self, chassis_id, params=''):
        chassis_show = self.ironic('chassis-show',
                                   params='{0} {1}'.format(chassis_id, params))
        return utils.get_dict_from_output(chassis_show)

    def update_chassis(self, chassis_id, operation, params=''):
        updated_chassis = self.ironic(
            'chassis-update',
            params='{0} {1} {2}'.format(chassis_id, operation, params))
        return utils.get_dict_from_output(updated_chassis)

    def delete_port(self, port_id, ignore_exceptions=False):
        try:
            self.ironic('port-delete', params=port_id)
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def create_port(self,
                    node_id,
                    mac_address=None,
                    flags='',
                    params=''):

        if mac_address is None:
            mac_address = data_utils.rand_mac_address()

        port = self.ironic('port-create',
                           flags=flags,
                           params='--address {0} --node {1} {2}'
                           .format(mac_address, node_id, params))
        if not port:
            self.fail('Ironic port has not been created!')

        return utils.get_dict_from_output(port)

    def list_ports(self, params=''):
        return self.ironic('port-list', params=params)

    def show_port(self, port_id, params=''):
        port_show = self.ironic('port-show', params='{0} {1}'
                                .format(port_id, params))
        return utils.get_dict_from_output(port_show)

    def get_uuids_from_port_list(self):
        port_list = self.list_ports()
        return [x['UUID'] for x in port_list]

    def update_port(self, port_id, operation, flags='', params=''):
        updated_port = self.ironic('port-update',
                                   flags=flags,
                                   params='{0} {1} {2}'
                                   .format(port_id, operation, params))
        return utils.get_dict_from_output(updated_port)

    def create_portgroup(self, node_id, params=''):
        """Create a new portgroup."""
        portgroup = self.ironic('portgroup-create',
                                flags=self.pg_api_ver,
                                params='--node {0} {1}'
                                .format(node_id, params))
        if not portgroup:
            self.fail('Ironic portgroup failed to create!')
        portgroup = utils.get_dict_from_output(portgroup)
        self.addCleanup(self.delete_portgroup, portgroup['uuid'],
                        ignore_exceptions=True)
        return portgroup

    def delete_portgroup(self, portgroup_id, ignore_exceptions=False):
        """Delete a port group."""
        try:
            self.ironic('portgroup-delete',
                        flags=self.pg_api_ver,
                        params=portgroup_id)
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def list_portgroups(self, params=''):
        """List the port groups."""
        return self.ironic('portgroup-list',
                           flags=self.pg_api_ver,
                           params=params)

    def show_portgroup(self, portgroup_id, params=''):
        """Show detailed information about a port group."""
        portgroup_show = self.ironic('portgroup-show',
                                     flags=self.pg_api_ver,
                                     params='{0} {1}'
                                     .format(portgroup_id, params))
        return utils.get_dict_from_output(portgroup_show)

    def update_portgroup(self, portgroup_id, op, params=''):
        """Update information about a port group."""
        updated_portgroup = self.ironic('portgroup-update',
                                        flags=self.pg_api_ver,
                                        params='{0} {1} {2}'
                                        .format(portgroup_id, op, params))
        return utils.get_dict_from_output(updated_portgroup)

    def get_portgroup_uuids_from_portgroup_list(self):
        """Get UUIDs from list of port groups."""
        portgroup_list = self.list_portgroups()
        return [x['UUID'] for x in portgroup_list]

    def portgroup_port_list(self, portgroup_id, params=''):
        """List the ports associated with a port group."""
        return self.ironic('portgroup-port-list', flags=self.pg_api_ver,
                           params='{0} {1}'.format(portgroup_id, params))



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.tests.functional.osc.v1 import base


class ProvisionStateTests(base.TestCase):
    """Functional tests for baremetal node provision state commands."""

    def setUp(self):
        super(ProvisionStateTests, self).setUp()
        self.node = self.node_create()

    def test_deploy_rebuild_undeploy_manage(self):
        """Deploy, rebuild and undeploy node.

        Test steps:
        1) Create baremetal node in setUp.
        2) Check initial "enroll" provision state.
        3) Set baremetal node "manage" provision state.
        4) Check baremetal node provision_state field value is "manageable".
        5) Set baremetal node "provide" provision state.
        6) Check baremetal node provision_state field value is "available".
        7) Set baremetal node "deploy" provision state.
        8) Check baremetal node provision_state field value is "active".
        9) Set baremetal node "rebuild" provision state.
        10) Check baremetal node provision_state field value is "active".
        11) Set baremetal node "undeploy" provision state.
        12) Check baremetal node provision_state field value is "available".
        13) Set baremetal node "manage" provision state.
        14) Check baremetal node provision_state field value is "manageable".
        15) Set baremetal node "provide" provision state.
        16) Check baremetal node provision_state field value is "available".
        """
        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("enroll", show_prop["provision_state"])

        # manage
        self.openstack('baremetal node manage {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("manageable", show_prop["provision_state"])

        # provide
        self.openstack('baremetal node provide {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("available", show_prop["provision_state"])

        # deploy
        self.openstack('baremetal node deploy {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("active", show_prop["provision_state"])

        # rebuild
        self.openstack('baremetal node rebuild {0}'.format(self.node['uuid']))

        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("active", show_prop["provision_state"])

        # undeploy
        self.openstack('baremetal node undeploy {0}'.format(self.node['uuid']))

        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("available", show_prop["provision_state"])

        # manage
        self.openstack('baremetal node manage {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("manageable", show_prop["provision_state"])

        # provide back
        self.openstack('baremetal node provide {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ["provision_state"])
        self.assertEqual("available", show_prop["provision_state"])



# Copyright (c) 2015 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


def get_dict_from_output(output):
    """Parse list of dictionaries, return a dictionary.

    :param output: list of dictionaries
    """
    obj = {}
    for item in output:
        obj[item['Property']] = str(item['Value'])
    return obj


def get_object(object_list, object_value):
    """Get Ironic object by value from list of Ironic objects.

    :param object_list: the output of the cmd
    :param object_value: value to get
    """
    for obj in object_list:
        if object_value in obj.values():
            return obj



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from ironicclient.tests.functional.osc.v1 import base


class PowerStateTests(base.TestCase):
    """Functional tests for baremetal node power state commands."""

    def setUp(self):
        super(PowerStateTests, self).setUp()
        self.node = self.node_create()

    def test_off_reboot_on(self):
        """Reboot node from Power OFF state.

        Test steps:
        1) Create baremetal node in setUp.
        2) Set node Power State OFF as precondition.
        3) Call reboot command for baremetal node.
        4) Check node Power State ON in node properties.
        """
        self.openstack('baremetal node power off {0}'
                       .format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ['power_state'])
        self.assertEqual('power off', show_prop['power_state'])

        self.openstack('baremetal node reboot {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ['power_state'])
        self.assertEqual('power on', show_prop['power_state'])

    def test_on_reboot_on(self):
        """Reboot node from Power ON state.

        Test steps:
        1) Create baremetal node in setUp.
        2) Set node Power State ON as precondition.
        3) Call reboot command for baremetal node.
        4) Check node Power State ON in node properties.
        """
        self.openstack('baremetal node power on {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ['power_state'])
        self.assertEqual('power on', show_prop['power_state'])

        self.openstack('baremetal node reboot {0}'.format(self.node['uuid']))
        show_prop = self.node_show(self.node['uuid'], ['power_state'])
        self.assertEqual('power on', show_prop['power_state'])



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from ironicclient.tests.functional.osc.v1 import base


class BaremetalPortTests(base.TestCase):
    """Functional tests for baremetal port commands."""

    def setUp(self):
        super(BaremetalPortTests, self).setUp()
        self.node = self.node_create()
        self.port = self.port_create(self.node['uuid'])

    def test_list(self):
        """Check baremetal port list command.

        Test steps:
        1) Create baremetal port in setUp.
        2) List baremetal ports.
        3) Check port address and UUID in ports list.
        """
        port_list = self.port_list()
        self.assertIn(self.port['address'],
                      [port['Address'] for port in port_list])
        self.assertIn(self.port['uuid'],
                      [port['UUID'] for port in port_list])

    def test_show_uuid(self):
        """Check baremetal port show command with UUID.

        Test steps:
        1) Create baremetal port in setUp.
        2) Show baremetal port calling it by UUID.
        3) Check port fields in output.
        """
        port = self.port_show(self.port['uuid'])
        self.assertEqual(self.port['address'], port['address'])
        self.assertEqual(self.port['uuid'], port['uuid'])
        self.assertEqual(self.port['node_uuid'], self.node['uuid'])

    def test_show_addr(self):
        """Check baremetal port show command with address.

        Test steps:
        1) Create baremetal port in setUp.
        2) Show baremetal port calling it by address.
        3) Check port fields in output.
        """
        port = self.port_show(
            uuid='', params='--address {}'.format(self.port['address']))
        self.assertEqual(self.port['address'], port['address'])
        self.assertEqual(self.port['uuid'], port['uuid'])
        self.assertEqual(self.port['node_uuid'], self.node['uuid'])

    def test_delete(self):
        """Check baremetal port delete command.

        Test steps:
        1) Create baremetal port in setUp.
        2) Delete baremetal port by UUID.
        3) Check that port deleted successfully and not in list.
        """
        output = self.port_delete(self.port['uuid'])
        self.assertIn('Deleted port {0}'.format(self.port['uuid']), output)
        port_list = self.port_list()
        self.assertNotIn(self.port['address'],
                         [port['Address'] for port in port_list])
        self.assertNotIn(self.port['uuid'],
                         [port['UUID'] for port in port_list])

    def test_set_unset_extra(self):
        """Check baremetal port set and unset commands.

        Test steps:
        1) Create baremetal port in setUp.
        2) Set extra data for port.
        3) Check that baremetal port extra data was set.
        4) Unset extra data for port.
        5) Check that baremetal port extra data was unset.
        """
        extra_key = 'ext'
        extra_value = 'testdata'
        self.openstack('baremetal port set --extra {0}={1} {2}'
                       .format(extra_key, extra_value, self.port['uuid']))

        show_prop = self.port_show(self.port['uuid'], ['extra'])
        self.assertEqual(extra_value, show_prop['extra'][extra_key])

        self.openstack('baremetal port unset --extra {0} {1}'
                       .format(extra_key, self.port['uuid']))

        show_prop = self.port_show(self.port['uuid'], ['extra'])
        self.assertNotIn(extra_key, show_prop['extra'])

    def test_port_create_with_portgroup(self):
        """Create port with specific port group UUID.

        Test steps:
        1) Create node in setUp().
        2) Create a port group.
        3) Create a port with specified port group.
        4) Check port properties for portgroup_uuid.
        """
        api_version = ' --os-baremetal-api-version 1.24'
        port_group = self.port_group_create(self.node['uuid'],
                                            params=api_version)
        port = self.port_create(
            self.node['uuid'],
            params='--port-group {0} {1}'.format(port_group['uuid'],
                                                 api_version))
        self.assertEqual(port_group['uuid'], port['portgroup_uuid'])
